/+
  Christoph Singewald  mail: christoph@singewald.at
  A scanner sample  using re2c version 0.12.1 (http://re2c.org)

  Commands:
  	re2c -i lexer.re > lexer.d
	dmd  lexer.d
  Important notes:

    * Use "re2c:yych:conversion=0" otherwise re2c uses a c-style cast.
    * The -i swtich is important to prevent re2c printing "#line" tags

+/
module lexer;

import std.file;
import std.stdio;

const int TOK_EOI       =  0;  // end of input
const int TOK_NUMBER    =  1;
const int TOK_IDENT     =  2;
const int TOK_VARIABLE  = 3;
const int TOK_ERROR     = -99;

/*!re2c
LETTER          = [a-zA-Z];
DIGIT	        = [0-9];
NUMBER			= DIGIT+;
IDENT  		    = [a-zA-Z0-9_']*;
VARIABLE  	    = "$"[a-zA-Z0-9'_']*;
WS				= [\t\r\b ]*;
COMMENT			= ["%" "#"] .*;
NEWLINE			= [\n];
*/

class Scanner
{
   protected:
		char* ptr;
		char* sptr;
		char* start;

	    int line = 1;

	    char[] data;
   public:

     this()
     {
       ptr=null;
       sptr=null;
       start = null;
     }
     void setInput(char[] input)
     {
     	data=input;
     	ptr = &data[0];
		sptr = null;

     }

     bool eof()
     {
     	return ptr>&data[data.length-1];
     }

     int currentLine()
     {
     	return line;
     }

     char[] currentToken()
     {
     	return start[0..(ptr-start)];
     }

 	 int scan()
	{
		start = ptr;
		scan:
		  if(eof()) return TOK_EOI;
		/*!re2c
				re2c:define:YYCTYPE  = "char";
				re2c:define:YYCURSOR = ptr;
				re2c:yyfill:enable   = 0;
				re2c:yych:conversion = 1;
				re2c:indent:top      = 1;
				re2c:yych:conversion = 0 ;
				re2c:define:YYMARKER = sptr;
					[0-9]+          { return TOK_NUMBER;}
					WS				{ start++; goto scan; }
					COMMENT			{ start=ptr; goto scan; }
					NEWLINE			{ line++; start++; goto scan; }
					IDENT			{ return TOK_IDENT; }
					VARIABLE		{ return TOK_VARIABLE; }
					. 				{ return TOK_ERROR;}

		*/
		}
}

int main(char[][] args){

    if(args.length<1) {
    	writefln("Too less arguments");
    	return 1;
    }

    char[]  input = cast(char[])std.file.read(args[1]);
	writefln("input:");
	writefln(input);

	Scanner s = new Scanner();
	s.setInput(input);
	int tok = 0;
	writefln("Start scanning ... ");
 	while(true)
	{
		tok=s.scan();
		writef(s.currentToken());
		writef(" " ~ std.string.toString(tok));
		writefln(" in line: " ~ std.string.toString(s.currentLine()));

		if(tok==TOK_ERROR) {
			writefln("Unexpected token in line: " ~ std.string.toString(s.currentLine()));
			break;
		}
		if(s.eof())
		   break;
	}
    return 0;

}

